from datetime import date

from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.mixins import PermissionRequiredMixin, LoginRequiredMixin
from django.contrib.messages.views import SuccessMessageMixin
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render, redirect
from django.urls import reverse_lazy
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django.views.generic import UpdateView
from django.db.models import Q, F
from django.utils.timezone import datetime
from django.contrib import messages
from ..forms import EventModerateForm
from ..models import Event


class EventModerateView(
    SuccessMessageMixin,
    PermissionRequiredMixin,
    LoginRequiredMixin,
    UpdateView,
):
    model = Event
    permission_required = "agenda_culturel.change_event"
    template_name = "agenda_culturel/event_form_moderate.html"
    form_class = EventModerateForm

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["is_moderation_expert"] = (
            self.request.user.userprofile.is_moderation_expert
        )
        return kwargs

    def get_success_message(self, cleaned_data):
        txt = (
            _(" A message has been sent to the person who proposed the event.")
            if hasattr(self, "with_msg") and self.with_msg
            else ""
        )
        return mark_safe(
            _('The event <a href="{}">{}</a> has been moderated with success.').format(
                self.object.get_absolute_url(), self.object.title
            )
            + txt
        )

    def get_next_event(start_day, start_time, opk):
        # select non moderated events
        qs = Event.objects.filter(moderated_date__isnull=True)

        # select events after the current one
        if start_time:
            qs = qs.filter(
                Q(start_day__gt=start_day)
                | (
                    Q(start_day=start_day)
                    & (Q(start_time__isnull=True) | Q(start_time__gt=start_time))
                )
            )
        else:
            qs = qs.filter(Q(start_day__gte=start_day) & ~Q(pk=opk))

        # get only possibly representative events
        qs = qs.filter(
            Q(other_versions__isnull=True)
            | Q(other_versions__representative=F("pk"))
            | Q(other_versions__representative__isnull=True)
        )

        # remove trash events
        qs = qs.filter(~Q(status=Event.STATUS.TRASH))

        # sort by datetime
        qs = qs.order_by("start_day", "start_time")

        return qs.first()

    def get_object(self, queryset=None):
        event = super().get_object(queryset)
        if event.status == Event.STATUS.DRAFT:
            event.status = Event.STATUS.PUBLISHED
        return event

    def post(self, request, *args, **kwargs):
        try:
            return super().post(request, args, kwargs)
        except Http404:
            return HttpResponseRedirect(
                reverse_lazy("error_next_event", args=[self.object.pk])
            )

    def form_valid(self, form):
        form.instance.set_no_modification_date_changed()
        form.instance.set_in_moderation_process()
        form.instance.set_processing_user(self.request.user)
        self.with_msg = form.instance.notify_if_required(self.request)
        return super().form_valid(form)

    def get_success_url(self):
        if "save_and_next" in self.request.POST:
            return reverse_lazy("moderate_event_next", args=[self.object.pk])
        elif "save_and_edit_local" in self.request.POST:
            return reverse_lazy("edit_event", args=[self.object.get_local_version().pk])
        else:
            return self.object.get_absolute_url()

    def dispatch(self, request, *args, **kwargs):
        self.object = self.get_object()

        msg = False
        if (
            self.object.other_versions is not None
            and self.object.other_versions.nb_duplicated() > 1
        ):
            representative = self.object.other_versions.representative
            if representative is not None:
                if representative != self.object:
                    msg = True
                    messages.warning(
                        request,
                        mark_safe(
                            _(
                                "There are several versions of this event, and you're moderating a version that isn't the one being promoted. Most of the time, you don't need to do this, just move on to the next one."
                            )
                        ),
                    )
            else:
                msg = True
                messages.warning(
                    request,
                    mark_safe(
                        _(
                            'This event exists in several versions, and has no highlighted version. You should <a href="{}">correct this situation</a> before moderating only the selected event.'
                        ).format(
                            reverse_lazy(
                                "fix_duplicate",
                                kwargs={"pk": self.object.other_versions.pk},
                            )
                        )
                    ),
                )

        issues = self.object.get_publication_issues()
        print([str(i) for i in issues])
        if len(issues) > 0 and not msg:
            if self.object.pure_import:
                lv = self.object.get_local_version()
                if lv is None:
                    messages.warning(
                        request,
                        mark_safe(
                            _(
                                "This event is synchronized to a source, and has properties that prevent it from being published as is ({}). We've redirected your moderation to event editing to correct these problems before publication."
                            ).format(", ".join([str(i) for i in issues]))
                        ),
                    )
                    # clone edit
                    return redirect(
                        reverse_lazy("clone_edit", kwargs={"pk": self.object.pk})
                    )
                else:
                    # msg is False, thus representative = self.object. Strange configuration. Anyway...
                    issues_lv = lv.get_publication_issues()
                    if len(issues_lv) == 0:
                        messages.warning(
                            request,
                            mark_safe(
                                _(
                                    'The event is synchronized to a source and is not publishable as is ({}), but there is <a href="{}">local version</a> that is publishable. You may wish to <a href="{}">modify the highlighted event</a>.'
                                ).format(
                                    ", ".join([str(i) for i in issues]),
                                    lv.get_absolute_url(),
                                    reverse_lazy("fix_duplicate"),
                                    kwargs={"pk": self.object.other_versions.pk},
                                )
                            ),
                        )
                    else:
                        messages.warning(
                            request,
                            mark_safe(
                                _(
                                    'The event is synchronized on a source and is not publishable as is ({}), and the <a href="{}">local version</a> is not publishable ({}). You should first <a href="{}">fix it</a>, then <a href="{}">modify the event displayed</a>.'
                                ).format(
                                    ", ".join([str(i) for i in issues]),
                                    lv.get_absolute_url(),
                                    ", ".join([str(i) for i in issues_lv]),
                                    reverse_lazy("edit_event", kwargs={"pk": lv.pk}),
                                    reverse_lazy(
                                        "fix_duplicate",
                                        kwargs={"pk": self.object.other_versions.pk},
                                    ),
                                )
                            ),
                        )
            else:
                messages.info(
                    request,
                    mark_safe(
                        _(
                            "Simple moderation is not available as the event is not publishable in its current state ({}). We therefore switch to editing mode."
                        ).format(", ".join([str(i) for i in issues]))
                    ),
                )
                return redirect(
                    reverse_lazy("edit_event", kwargs={"pk": self.object.pk})
                )
        return super().dispatch(request, *args, **kwargs)


class EventModerateNextView(EventModerateView):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["pred"] = self.kwargs["pred"]
        return context


class EventModerateBackView(EventModerateView):
    def get_object(self, queryset=None):
        pred = Event.objects.filter(pk=self.kwargs["pred"]).first()
        pred.free_modification_lock(self.request.user)
        return super().get_object(queryset)


class EventModerateForceView(EventModerateView):
    def get_object(self, queryset=None):
        event = super().get_object(queryset)
        event.free_modification_lock(self.request.user, False)
        return super().get_object(queryset)


@login_required(login_url="/accounts/login/")
@permission_required("agenda_culturel.change_event")
def moderate_from_now(request):
    now = datetime.now()
    obj = EventModerateView.get_next_event(now.date(), now.time(), None)
    return HttpResponseRedirect(reverse_lazy("moderate_event", args=[obj.pk]))


@login_required(login_url="/accounts/login/")
@permission_required("agenda_culturel.change_event")
def moderate_event_next(request, pk):
    # current event
    obj = Event.objects.filter(pk=pk).first()
    # free lock
    obj.free_modification_lock(request.user)
    start_day = obj.start_day
    start_time = obj.start_time

    next_obj = EventModerateView.get_next_event(start_day, start_time, pk)
    if next_obj is None:
        return render(
            request,
            "agenda_culturel/event_next_error_message.html",
            {"pk": pk, "object": obj},
        )
    else:
        return HttpResponseRedirect(
            reverse_lazy("moderate_event_step", args=[next_obj.pk, obj.pk])
        )


@login_required(login_url="/accounts/login/")
@permission_required("agenda_culturel.change_event")
def moderate_from_date(request, y, m, d):
    d = date(y, m, d)
    obj = EventModerateView.get_next_event(d, None, None)
    return HttpResponseRedirect(reverse_lazy("moderate_event", args=[obj.pk]))
